# -*- coding:utf-8 -*-
biosPostEventStr_EN = {}
biosPostEventStr_EN[0] = {}
biosPostEventStr_EN[1] = {}
biosPostEventStr_EN[2] = biosPostEventStr_EN[1]

biosPostEventStr_EN[0][0] = "Unspecified."
biosPostEventStr_EN[0][1] = "No system memory is physically installed in the system."
biosPostEventStr_EN[0][2] = "No usable system memory, all installed memory has experienced an unrecoverable failure."
biosPostEventStr_EN[0][3] = "Unrecoverable hard-disk/ATAPI/IDE device failure."
biosPostEventStr_EN[0][4] = "Unrecoverable system-board failure."
biosPostEventStr_EN[0][5] = "Unrecoverable diskette subsystem failure."
biosPostEventStr_EN[0][6] = "Unrecoverable hard-disk controller failure."
biosPostEventStr_EN[0][7] = "Unrecoverable PS/2 or USB keyboard failure."
biosPostEventStr_EN[0][8] = "Removable boot media not found."
biosPostEventStr_EN[0][9] = "Unrecoverable video controller failure."
biosPostEventStr_EN[0][10] = "No video device detected."
biosPostEventStr_EN[0][11] = "Firmware (BIOS) ROM corruption detected."
biosPostEventStr_EN[0][12] = "CPU voltage mismatch (processors that share same supply have mismatched voltage requirements)."
biosPostEventStr_EN[0][13] = "CPU speed matching failure."


biosPostEventStr_EN[1][0] = "Unspecified."
biosPostEventStr_EN[1][1] = "Memory initialization."
biosPostEventStr_EN[1][2] = "Hard-disk initialization."
biosPostEventStr_EN[1][3] = "Secondary processor(s) initialization."
biosPostEventStr_EN[1][4] = "User authentication."
biosPostEventStr_EN[1][5] = "User-initiated system setup."
biosPostEventStr_EN[1][6] = "USB resource configuration."
biosPostEventStr_EN[1][7] = "PCI resource configuration."
biosPostEventStr_EN[1][8] = "Option ROM initialization."
biosPostEventStr_EN[1][9] = "Video initialization."
biosPostEventStr_EN[1][10] = "Cache initialization."
biosPostEventStr_EN[1][11] = "SM Bus initialization."
biosPostEventStr_EN[1][12] = "Keyboard controller initialization."
biosPostEventStr_EN[1][13] = "Embedded controller/management controller initialization."
biosPostEventStr_EN[1][14] = "Docking station attachment."
biosPostEventStr_EN[1][15] = "Enabling docking station."
biosPostEventStr_EN[1][16] = "Docking station ejection."
biosPostEventStr_EN[1][17] = "Disabling docking station."
biosPostEventStr_EN[1][18] = "Calling operating system wake-up vector."
biosPostEventStr_EN[1][19] = "Starting operating system boot process."
biosPostEventStr_EN[1][20] = "Baseboard or motherboard initialization."
biosPostEventStr_EN[1][21] = "reserved."
biosPostEventStr_EN[1][22] = "Floppy initialization."
biosPostEventStr_EN[1][23] = "Keyboard test."
biosPostEventStr_EN[1][24] = "Pointing device test."
biosPostEventStr_EN[1][25] = "Primary processor initialization."

biosPostEventStr = {}
biosPostEventStr["en"] = biosPostEventStr_EN
